#!/bin/bash
DTD=2.3.dtd
XSD=2.3.xsd
XML=2.3.xml
TD=2.3

function die {
  echo -e "ERROR: $@"
  exit 1
}

function check_eq {
  [[ "$1" = "$2" ]] || die "ERROR: Expected:\n$2\nAnd got instead:\n$1"
}

echo "Testing the XML"
[[ -f "$XML" ]] || die "'$XML' not found."
check_eq "$(xsltproc 2.3.test.xslt $XML | sort -n | awk '{print}' 'ORS= ')"\
         "droit droit droit droit gauche gauche gauche gauche 1 3 4 6 7 8 10 13 14 "
echo "'$XML' *seems* to be as expected."

echo "Testing the XML with your DTD"
[[ -f "$DTD" ]] || die "'$DTD' not found."
xmllint --noout --dtdvalid $DTD $XML || die "'$XML' is not valid with your DTD '$DTD'"
echo "'$XML' is valid with '$DTD'."

[[ -f ${TD}.valid1.xml && -f ${TD}.invalid0_1.xml ]] ||\
  die "You need to run this script in the same directory as the ${TD}.valid*.xml files.";

echo ""
echo "Testing the DTD (level 0 and 1)...."
for i in ${TD}.valid*.xml; do
  if xmllint --dtdvalid $DTD --noout $i; then
    echo "$i is valid as expected"
  else
    die "$i should be valid under the DTD!"
  fi
done

for i in ${TD}.invalid{0,1,2}_*.xml; do
  if xmllint --dtdvalid $DTD --noout $i 2> /dev/null; then
    die "$i was valid -- but it should be invalid under the DTD!"
  else
    echo "$i was invalid as expected"
  fi
done
echo "SUCCESS! DTD is OK."

echo "Testing the XML with your XSD"
[[ -f "$XSD" ]] || die "'$XSD' not found."
xmllint --noout $XSD $XML || die "'$XML' is not valid with '$XSD'"
echo "'$XML' is valid with '$XSD'."

echo ""
echo "Testing the Schema (baseline)...."
for i in ${TD}.valid*.xml; do
  if xmllint --schema $XSD --noout $i; then
    echo -n ""
  else
    die "$i should be valid under the XML schema!"
  fi
done
echo "SUCCESS!"

function TestSchema() {
  echo ""
  echo "Testing the schema, level $1: $2."
  for i in ${TD}.invalid${1}_*.xml; do
    if xmllint --schema $XSD --noout $i 2> /dev/null; then
      die "$i was valid -- but it should be invalid under the XML schema!"
    else
      echo "$i was invalid as expected"
    fi
  done
  echo "SUCCESS!"
}

TestSchema 0 "basic"

echo "YES! YOU MADE IT!!!"
