#!/bin/bash
DTD=2.3.dtd
XSD=2.4.xsd
XML=2.3.xml
TD=2.3

function die {
  echo "ERROR: $@"
  exit 1
}

[[ -f "$DTD" ]] || die "'$DTD' not found."
[[ -f ${TD}.valid1.xml && -f ${TD}.invalid0_1.xml ]] ||\
  die "You need to run this script in the same directory as the ${TD}.valid*.xml files.";

echo ""
echo "Testing the DTD (level 0 and 1)...."
for i in ${TD}.valid*.xml; do
  if xmllint --dtdvalid $DTD --noout $i; then
    echo "$i is valid as expected"
  else
    die "$i should be valid under the DTD!"
  fi
done

for i in ${TD}.invalid{0,1,2}_*.xml; do
  if xmllint --dtdvalid $DTD --noout $i 2> /dev/null; then
    die "$i was valid -- but it should be invalid under the DTD!"
  else
    echo "$i was invalid as expected"
  fi
done
echo "SUCCESS! DTD is OK."

echo "Testing the XML with your DTD"
[[ -f "$XML" ]] || die "'$XML' not found."
xmllint --noout --dtdvalid $DTD $XML || die "'$XML' is not valid with your DTD '$DTD'"
echo "'$XML' is valid with '$DTD'."

echo "Testing the XML with your XSD"
[[ -f "$XSD" ]] || die "'$XSD' not found."
xmllint --noout $XSD $XML || die "'$XML' is not valid with '$XSD'"
echo "'$XML' is valid with '$XSD'."

echo ""
echo "Testing the Schema (baseline)...."
for i in ${TD}.valid*.xml; do
  if xmllint --schema $XSD --noout $i; then
    echo -n ""
  else
    die "$i should be valid under the XML schema!"
  fi
done
echo "SUCCESS!"

function TestSchema() {
  echo ""
  echo "Testing the schema, level $1: $2."
  for i in ${TD}.invalid${1}_*.xml; do
    if xmllint --schema $XSD --noout $i 2> /dev/null; then
      die "$i was valid -- but it should be invalid under the XML schema!"
    else
      echo "$i was invalid as expected"
    fi
  done
  echo "SUCCESS!"
}

TestSchema 0 "basic"
TestSchema 1 "noeud.id must be unique"
TestSchema 2 "noeud.gauche and noeud.droit must refer to an existing noeud.id."

echo "YES! YOU MADE IT!!!"
